'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _cliEngineCommand = require('cli-engine-command');

var _cliEngineCommand2 = _interopRequireDefault(_cliEngineCommand);

var _http = require('cli-engine-command/lib/http');

var _http2 = _interopRequireDefault(_http);

var _netrcParser = require('netrc-parser');

var _netrcParser2 = _interopRequireDefault(_netrcParser);

var _output = require('cli-engine-command/lib/output');

var _output2 = _interopRequireDefault(_output);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _plugins = require('./plugins');

var _plugins2 = _interopRequireDefault(_plugins);

var _heroku = require('cli-engine-command/lib/heroku');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class AnalyticsCommand {

  constructor(options) {
    this.out = options.out;
    this.plugins = options.plugins || new _plugins2.default(this.out);
    this.http = new _http2.default(this.out);
    this.config = options.config;
  }

  _initialAnalyticsJSON() {
    return {
      schema: 1,
      commands: []
    };
  }

  record(Command) {
    try {
      let plugin = this.plugins.list().find(p => p.findCommand(Command.id));
      if (!plugin) {
        this.out.debug('no plugin found for analytics');
        return;
      }

      if (!this.user) return;

      let analyticsJSON;
      if (this._existsJSON()) {
        try {
          analyticsJSON = this._readJSON();
        } catch (err) {
          this.out.debug(err);
        }
      }

      if (!analyticsJSON) {
        analyticsJSON = this._initialAnalyticsJSON();
      }

      analyticsJSON.commands.push({
        command: Command.id,
        version: this.config.version,
        plugin: plugin.name,
        plugin_version: plugin.version,
        os: this.config.platform,
        valid: true,
        language: 'node'
      });

      this._writeJSON(analyticsJSON);
    } catch (err) {
      this.out.debug(err);
    }
  }

  async submit() {
    try {
      let user = this.user;
      if (!user) return;

      const local = this._readJSON();
      if (local.commands.length === 0) return;

      const body = {
        schema: local.schema,
        commands: local.commands,
        user: user,
        install: this.config.install,
        cli: this.config.name
      };

      await this.http.post(this.url, { body });

      local.commands = [];
      this._writeJSON(local);
    } catch (err) {
      this.out.debug(err);
    }
  }

  get url() {
    return process.env['CLI_ENGINE_ANALYTICS_URL'] || 'https://cli-analytics.heroku.com/record';
  }

  get analyticsPath() {
    return _path2.default.join(this.config.cacheDir, 'analytics.json');
  }

  get usingHerokuAPIKey() {
    return !!(process.env['HEROKU_API_KEY'] && process.env['HEROKU_API_KEY'].length > 0);
  }

  get netrcLogin() {
    let netrc = new _netrcParser2.default();
    return netrc.machines[_heroku.vars.apiHost].login;
  }

  get user() {
    if (this.config.skipAnalytics || this.usingHerokuAPIKey) return;
    return this.netrcLogin;
  }

  _existsJSON() {
    return _fsExtra2.default.existsSync(this.analyticsPath);
  }

  _readJSON() {
    return _fsExtra2.default.readJSONSync(this.analyticsPath);
  }

  _writeJSON(analyticsJSON) {
    return _fsExtra2.default.writeJSONSync(this.analyticsPath, analyticsJSON);
  }
}
exports.default = AnalyticsCommand;