'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _cliEngineCommand = require('cli-engine-command');

var _cliEngineCommand2 = _interopRequireDefault(_cliEngineCommand);

var _updater = require('../updater');

var _updater2 = _interopRequireDefault(_updater);

var _update = require('./plugins/update');

var _update2 = _interopRequireDefault(_update);

var _analytics = require('../analytics');

var _analytics2 = _interopRequireDefault(_analytics);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Update extends _cliEngineCommand2.default {

  async run() {
    this.updater = new _updater2.default(this.out);
    if (this.config.updateDisabled) this.out.warn(this.config.updateDisabled);else {
      this.out.action.start(`${this.config.name}: Updating CLI`);
      let channel = this.argv[0] || this.config.channel;
      let manifest = await this.updater.fetchManifest(channel);
      if (this.config.version === manifest.version && channel === this.config.channel) {
        this.out.action.stop(`already on latest version: ${this.config.version}`);
      } else {
        this.out.action.start(`${this.config.name}: Updating CLI to ${this.out.color.green(manifest.version)}${channel === 'stable' ? '' : ' (' + this.out.color.yellow(channel) + ')'}`);
        await this.updater.update(manifest);
        this.out.action.stop();
      }
    }
    let analytics = new _analytics2.default({ out: this.out, config: this.config });
    await analytics.submit();
    await _update2.default.run({ config: this.config });
  }
}
exports.default = Update;
Update.topic = 'update';
Update.args = [{ name: 'channel', optional: true }];