'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.convertFromV5 = convertFromV5;
exports.convertFlagsFromV5 = convertFlagsFromV5;

var _cliEngineCommand = require('cli-engine-command');

var _cliEngineCommand2 = _interopRequireDefault(_cliEngineCommand);

var _app = require('cli-engine-command/lib/flags/app');

var _org = require('cli-engine-command/lib/flags/org');

var _org2 = _interopRequireDefault(_org);

var _heroku = require('cli-engine-command/lib/heroku');

var _heroku2 = _interopRequireDefault(_heroku);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function convertFromV5(c) {
  if (!c.topic) throw new Error('command has no topic');
  class V5 extends _cliEngineCommand2.default {

    run() {
      this.heroku = new _heroku2.default(this.out, { required: false });
      let flags = this.flags;
      let args = this.argv;
      if (!c.variableArgs) {
        // turn args into object v5 expects
        args = {};
        for (let i = 0; i < this.argv.length; i++) {
          args[this.constructor.args[i].name] = this.argv[i];
        }
      }
      const ctx = {
        supportsColor: this.out.color.enabled,
        auth: {},
        debug: this.config.debug,
        flags,
        args,
        app: flags.app,
        org: flags.org,
        config: this.config,
        apiUrl: _heroku.vars.apiUrl,
        herokuDir: this.config.cacheDir,
        apiToken: this.heroku.auth,
        apiHost: _heroku.vars.apiHost,
        gitHost: _heroku.vars.gitHost,
        httpGitHost: _heroku.vars.httpGitHost
      };
      ctx.auth.password = ctx.apiToken;
      return c.run(ctx);
    }
  }

  V5.topic = c.topic;
  V5.command = c.command;
  V5.description = c.description;
  V5.hidden = c.hidden;
  V5.args = c.args || [];
  V5.flags = convertFlagsFromV5(c.flags);
  V5.variableArgs = !!c.variableArgs;
  V5.help = c.help;
  if (c.needsApp || c.wantsApp) {
    V5.flags.app = (0, _app.AppFlag)({ required: !!c.needsApp });
    V5.flags.remote = (0, _app.RemoteFlag)();
  }
  if (c.needsOrg || c.wantsOrg) V5.flags.org = (0, _org2.default)({ required: !!c.needsOrg });
  return V5;
}

function convertFlagsFromV5(flags) {
  if (!flags) return {};
  if (!Array.isArray(flags)) return flags;
  return flags.reduce((flags, flag) => {
    let opts = {
      char: flag.char,
      description: flag.description,
      hidden: flag.hidden,
      required: flag.required,
      optional: flag.optional
    };
    flags[flag.name] = flag.hasValue ? (0, _cliEngineCommand.StringFlag)(opts) : (0, _cliEngineCommand.BooleanFlag)(opts);
    return flags;
  }, {});
}