'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.PluginPath = undefined;

require('cli-engine-config');

var _cliEngineCommand = require('cli-engine-command');

var _cliEngineCommand2 = _interopRequireDefault(_cliEngineCommand);

require('./cache');

var _legacy = require('./legacy');

var _namespaces = require('../namespaces');

var _namespaces2 = _interopRequireDefault(_namespaces);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class PluginPath {
  constructor(options) {
    this.out = options.output;
    this.path = options.path;
    this.type = options.type;
    this.tag = options.tag;

    this.config = this.out.config;
  }

  convertToCached() {
    let plugin = this.require();

    const getAliases = c => {
      let aliases = c.aliases || [];
      if (c.default) {
        this.out.warn(`default setting on ${c.topic} is deprecated`);
        aliases.push(c.topic);
      }
      return aliases;
    };

    if (!plugin.commands) throw new Error('no commands found');

    const commands = plugin.commands.map(c => ({
      id: c.command ? `${c.topic}:${c.command}` : c.topic,
      topic: c.topic,
      command: c.command,
      description: c.description,
      args: c.args,
      variableArgs: c.variableArgs,
      help: c.help,
      usage: c.usage,
      hidden: !!c.hidden,
      aliases: getAliases(c),
      flags: (0, _legacy.convertFlagsFromV5)(c.flags)
    }));
    const topics = (plugin.topics || (plugin.topic ? [plugin.topic] : [])).map(t => ({
      topic: t.topic || t.name || '',
      description: t.description,
      hidden: !!t.hidden
    }));

    for (let command of commands) {
      if (topics.find(t => t.topic === command.topic)) continue;
      topics.push({
        topic: command.topic,
        hidden: true
      });
    }

    const { name, version } = this.pjson();
    return { name, path: this.path, version, commands, topics };
  }

  undefaultTopic(t) {
    if (t.default) return t.default;
    return t;
  }

  undefaultCommand(c) {
    if (c.default && typeof c.default !== 'boolean') return c.default;
    return c;
  }

  require() {
    // flow$ignore
    let required = require(this.path);
    let plugin = {
      topic: required.topic && this.undefaultTopic(required.topic),
      topics: required.topics && required.topics.map(this.undefaultTopic),
      commands: required.commands && required.commands.map(this.undefaultCommand)
    };
    if (required.type === 'builtin' || /(\\|\/)(src|lib)(\\|\/)commands$/.test(this.path)) return plugin;
    return _namespaces2.default.namespacePlugin(plugin, this.path, this.config);
  }

  pjson() {
    if (this.type === 'builtin') {
      return { name: 'builtin', version: this.config.version };
    }

    // flow$ignore
    return require(_path2.default.join(this.path, 'package.json'));
  }
}

exports.PluginPath = PluginPath;