'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

require('cli-engine-config');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _rwlockfile = require('rwlockfile');

var _rwlockfile2 = _interopRequireDefault(_rwlockfile);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function fork(modulePath, args, options) {
  const { fork } = require('child_process');
  return new Promise((resolve, reject) => {
    fork(modulePath, args, options).on('error', reject).on('exit', code => resolve({ code }));
  });
}

class Yarn {

  constructor(output, cwd) {
    this.out = output;
    this.config = output.config;
    this.cwd = cwd;
  }

  get version() {
    return require('../../package.json')['cli-engine']['yarnDependency'];
  }
  get lockfile() {
    return _path2.default.join(this.config.cacheDir, 'yarn.lock');
  }
  get bin() {
    return _path2.default.join(__dirname, '..', '..', 'yarn', `yarn-${this.version}.js`);
  }

  async exec(args = []) {
    let options = {
      cwd: this.cwd,
      stdio: this.config.debug ? [0, 1, 2, 'ipc'] : [null, null, null, 'ipc']
    };

    this.out.debug(`${options.cwd}: ${this.bin} ${args.join(' ')}`);
    let unlock = await _rwlockfile2.default.write(this.lockfile);
    await fork(this.bin, args, options);
    await unlock();
  }
}
exports.default = Yarn;